pub mod is_prime;

use std::{time::SystemTime,thread};
use num_bigint::BigUint;
use crate::is_prime::is_prime::is_prime;

fn p(n: f64) -> f64 {
    let ln_n = n.ln();
    let ln_ln_n = ln_n.ln();
    let ln_ln_ln_n = ln_ln_n.ln();

    n * (
        ln_n + ln_ln_n - 1.0
        + (ln_ln_ln_n - 2.0) / ln_n
        - ((ln_ln_n).powi(2) - 6.0 * ln_ln_n + 11.0) / (2.0 * 2.0f64.log2() * ln_n)
        + ((ln_ln_n / ln_n).powi(3)) * (1.0 / ln_n)
    )
}


fn main() {
    /* 
    let mut buffer = String::new();
    let stdin = io::stdin();
    print!("please enter the number to check: ");
    io::stdout().flush().expect("could not flush");
    stdin.read_line(&mut buffer).expect("could not read from stdin");
    buffer = buffer.trim().to_owned();
    
    let number = BigUint::parse_bytes(&buffer.as_bytes(), 10).expect("expected positive number");

    if is_prime(number.clone()) {
        println!("\n{number} is a prime");
    } else {
        println!("\n{number} is not a prime");
    }
    */

    let sys_time = SystemTime::now();

    let n:f64 = 1_000_000f64;
    let nthprime_approx = p(n*1.01) as u128;
    const THREADS: u128 = 12;
    let workloads = nthprime_approx/THREADS;

    let mut primes: Vec<BigUint> = vec![];
    let mut joins = vec![];
    for i in 0..THREADS {
        let tprimes = thread::spawn(move || {
            let mut lprimes = vec![];
            let work_uint = BigUint::from(workloads);
            let mut number = BigUint::from(i)*&work_uint;
            let inc = BigUint::from(1u8);
            
            let max = BigUint::from(i+1)*work_uint;
            
            loop {
                number = number + &inc;
                if is_prime(&number) {
                    lprimes.push(number.clone());
                }
                if number == max {
                    break;
                }
            }
            lprimes
        });
        joins.push(tprimes);
    }

    loop {
        let mut njoins = vec![];
        for join in joins {
            if join.is_finished() {
                let tp = join.join();
                if let Ok(mut tprimes) = tp {
                    primes.append(&mut tprimes);
                }
            } else {
                njoins.push(join);
            }
        }
        joins = njoins;
        if joins.len()==0 {
            break;
        }
    }
    let removed = (primes.len()-n as usize).clamp(0, n as usize);
    if primes.len() > n as usize {
        for _ in 0..removed {
            primes.pop();
        }
    }

    let added = (n as usize-primes.len()).clamp(0, n as usize);
    if primes.len() < n as usize {
        let mut number = BigUint::from(THREADS)*BigUint::from(workloads);
        let inc = BigUint::from(1u8);
        loop {
            number = number + &inc;
            if is_prime(&number) {
                primes.push(number.clone());
            }
            if primes.len() == n as usize {
                break;
            }
        }
    }

    let new_sys_time = SystemTime::now();
    let difference = new_sys_time.duration_since(sys_time)
    .expect("Clock may have gone backwards");

    println!("{:?}", primes);

    println!("prime count: {}, expected: {}, primes added: {} | {}%",primes.len(),n,added,(added as f64)/primes.len() as f64*100.0);
    println!("last prime approx: {}",nthprime_approx);

    println!("And that's time: {difference:?}");


}
